import type { Permissions } from '$lib/permissions';
import type { Cookies } from '@sveltejs/kit';
import * as crypto from 'crypto';
import type { Admin } from '$lib/server/database';

const sessions: Map<string, { userId: number; permissions: Permissions }> = new Map();

export function addSession(user: { id: number; permissions: Permissions } | Admin): string {
	const session = crypto.randomBytes(16).toString('hex');
	sessions.set(session, { userId: user.id, permissions: user.permissions });
	return session;
}

export function getSession(session: string | Cookies, permissions?: number[]): Permissions | null {
	let sess: Permissions | null;
	if (typeof session == 'string') {
		sess = sessions.get(session)?.permissions || null;
	} else {
		const sessionId = session.get('session');
		sess = sessionId ? sessions.get(sessionId)?.permissions || null : null;
	}

	if (!sess) {
		return null;
	}
	for (const perm of permissions || []) {
		if ((sess.value & perm) == 0) {
			return null;
		}
	}
	return sess;
}

export function deleteSession(session: string | Cookies) {
	if (typeof session == 'string') {
		sessions.delete(session);
	} else {
		sessions.delete(session.get('session') || '');
	}
}

export function deleteAllUserSessions(userId: number) {
	for (const [id, details] of sessions.entries()) {
		if (details.userId == userId) {
			sessions.delete(id);
		}
	}
}
