import type { RequestHandler } from '@sveltejs/kit';
import { Permissions } from '$lib/permissions';
import { addSession, deleteAllUserSessions, deleteSession, getSession } from '$lib/server/session';
import { Admin } from '$lib/server/database';
import { env as publicEnv } from '$env/dynamic/public';

export const POST = (async ({ request, cookies }) => {
	if (getSession(cookies, [Permissions.AdminWrite]) == null) {
		return new Response(null, {
			status: 401
		});
	}

	const data = await request.json();
	const username = data['username'] as string | null;
	const password = data['password'] as string | null;
	const permissions = data['permissions'] as number | null;

	if (username == null || password == null || permissions == null) {
		return new Response(null, {
			status: 400
		});
	}

	const admin = await Admin.create({
		username: username,
		password: password,
		permissions: new Permissions(permissions)
	});

	return new Response(JSON.stringify(admin), {
		status: 201
	});
}) satisfies RequestHandler;

export const PATCH = (async ({ request, cookies }) => {
	if (getSession(cookies, [Permissions.AdminWrite]) == null) {
		return new Response(null, {
			status: 401
		});
	}

	const data = await request.json();
	const id = data['id'] as string | null;

	if (id == null) {
		return new Response(null, {
			status: 400
		});
	}

	const updatePayload: { [key: string]: any } = {};
	if (data['username']) updatePayload.username = data['username'];
	if (data['password']) updatePayload.password = data['password'];
	if (data['permissions']) updatePayload.permissions = data['permissions'];

	let user = await Admin.findOne({ where: { id: id } });
	if (!user) {
		return new Response(null, {
			status: 400
		});
	}
	user = await user.update(updatePayload);

	deleteSession(cookies);
	cookies.set('session', addSession(user), {
		path: `${publicEnv.PUBLIC_BASE_PATH}/admin`,
		maxAge: 60 * 60 * 24 * 90,
		httpOnly: true,
		secure: true
	});

	return new Response();
}) satisfies RequestHandler;

export const DELETE = (async ({ request, cookies }) => {
	if (getSession(cookies, [Permissions.AdminWrite]) == null) {
		return new Response(null, {
			status: 401
		});
	}

	const data = await request.json();
	const id = data['id'] as number | null;

	if (id == null) {
		return new Response(null, {
			status: 400
		});
	}

	await Admin.destroy({ where: { id: id } });
	deleteAllUserSessions(id);

	return new Response();
}) satisfies RequestHandler;
